﻿Imports System.Reflection
Imports Windows.UI.Xaml
Imports Windows.UI.Xaml.Controls

Namespace Charmed.Helpers
	''' <summary>
	''' Helper class for routing a ListView's ItemClicks to a method on the view model.
	''' </summary>
	Public NotInheritable Class ListViewHelper
		Public Shared ReadOnly ItemClickMethodNameProperty As DependencyProperty = DependencyProperty.RegisterAttached("ItemClickMethodName", GetType(String), GetType(ListViewHelper), New PropertyMetadata(Nothing, New PropertyChangedCallback(Sub(sender, args)
			Dim listView as ListViewBase = TryCast(sender, ListViewBase)
			If listView IsNot Nothing Then
				Dim TempListViewHelper As ListViewHelper = New ListViewHelper(listView)
			End If
		End Sub)))

		Public Sub New(ByVal listView As ListViewBase)
			AddHandler listView.ItemClick, Sub(sender, args)
				Dim methodName as String = GetItemClickMethodName(listView)

				Dim method as MethodInfo = listView.DataContext.GetType.GetTypeInfo.GetDeclaredMethod(methodName)
				Dim parms As System.Array = method.GetParameters

				If parms IsNot Nothing AndAlso parms.Length = 1 Then
					method.Invoke(listView.DataContext, { args.ClickedItem })
				Else
					method.Invoke(listView.DataContext, Nothing)
				End If
			End Sub
		End Sub

		Public Shared Sub SetItemClickMethodName(ByVal attached As DependencyObject, ByVal itemClickMethodName As String)
			attached.SetValue(ItemClickMethodNameProperty, itemClickMethodName)
		End Sub

		Public Shared Function GetItemClickMethodName(ByVal attached As DependencyObject) As String
			Return CStr(attached.GetValue(ItemClickMethodNameProperty))
		End Function
	End Class
End Namespace
