﻿Imports System.IO
Imports System.Text
Imports System.Threading.Tasks
Imports Windows.Storage
Imports Windows.Foundation

Namespace Charmed
    Public NotInheritable Class Storage
        Implements IStorage
        Private ReadOnly serializer As ISerializer

        Public Sub New(ByVal serializer As ISerializer)
            Me.serializer = serializer
        End Sub

        Public Async Function LoadAsync(Of T)(ByVal fileName As String) As Task(Of T) Implements IStorage.LoadAsync
            Dim file As StorageFile = TryCast(ApplicationData.Current.LocalFolder.CreateFileAsync(fileName, CreationCollisionOption.OpenIfExists), StorageFile)
            Dim serializedData As String = String.Empty
            Using stream As Stream = Await file.OpenStreamForReadAsync
                Using streamReader As StreamReader = New StreamReader(stream)
                    serializedData = Await streamReader.ReadToEndAsync
                End Using
            End Using

            Dim data As T = Nothing
            If Not String.IsNullOrWhiteSpace(serializedData) Then
                data = Me.serializer.Deserialize(Of T)(serializedData)
            End If

            Return data
        End Function

        Public Async Function SaveAsync(ByVal fileName As String, ByVal data As Object) As Task Implements IStorage.SaveAsync
            Dim file As StorageFile = TryCast(ApplicationData.Current.LocalFolder.CreateFileAsync(fileName, CreationCollisionOption.ReplaceExisting), StorageFile)

            Dim serializedData As String = Me.serializer.Serialize(data)

            Using Stream As Stream = Await file.OpenStreamForWriteAsync
                Dim fileBytes() As Byte = Encoding.UTF8.GetBytes(serializedData.ToCharArray)
                Await Stream.WriteAsync(fileBytes, 0, fileBytes.Length)
            End Using
        End Function
    End Class
End Namespace
